@@ -1,17 +1,17 @@
-from collections import OrderedDict
-from datetime import datetime
-from copy import copy
-import ruamel.yaml as yaml
 import os.path
 import string
-from warnings import warn
-from itertools import chain
 from abc import ABCMeta, abstractmethod
+from collections import OrderedDict
+from copy import copy
+from datetime import datetime
+from itertools import chain
+from warnings import warn
+
+import ruamel.yaml as yaml
 
-from ..utils import docval, getargs, popargs, get_docval, call_docval_func
 from .catalog import SpecCatalog
 from .spec import DatasetSpec, GroupSpec
-
+from ..utils import docval, getargs, popargs, get_docval, call_docval_func
 
 _namespace_args = [
     {'name': 'doc', 'type': str, 'doc': 'a description about what this namespace represents'},
@@ -230,7 +230,7 @@ class NamespaceCatalog:
             {'name': 'dataset_spec_cls', 'type': type,
              'doc': 'the class to use for dataset specifications', 'default': DatasetSpec},
             {'name': 'spec_namespace_cls', 'type': type,
-             'doc': 'the class to use for specification namespaces', 'default': SpecNamespace},)
+             'doc': 'the class to use for specification namespaces', 'default': SpecNamespace})
     def __init__(self, **kwargs):
         """Create a catalog for storing  multiple Namespaces"""
         self.__namespaces = OrderedDict()
@@ -327,6 +327,18 @@ class NamespaceCatalog:
             raise KeyError("'%s' not a namespace" % namespace)
         return spec_ns.get_hierarchy(data_type)
 
+    @docval({'name': 'namespace', 'type': str, 'doc': 'the name of the namespace containing the data_type'},
+            {'name': 'data_type', 'type': str, 'doc': 'the data_type to check'},
+            {'name': 'parent_data_type', 'type': str, 'doc': 'the potential parent data_type'},
+            returns="True if *data_type* is a sub `data_type` of *parent_data_type*, False otherwise", rtype=bool)
+    def is_sub_data_type(self, **kwargs):
+        '''
+        Return whether or not *data_type* is a sub `data_type` of *parent_data_type*
+        '''
+        ns, dt, parent_dt = getargs('namespace', 'data_type', 'parent_data_type', kwargs)
+        hier = self.get_hierarchy(ns, dt)
+        return parent_dt in hier
+
     @docval(rtype=tuple)
     def get_sources(self, **kwargs):
         '''
@@ -467,7 +479,10 @@ class NamespaceCatalog:
         to_load = list()
         for ns in namespaces:
             if ns['name'] in self.__namespaces:
-                warn("ignoring namespace '%s' because it already exists" % ns['name'])
+                if ns['version'] != self.__namespaces.get(ns['name'])['version']:
+                    # warn if the cached namespace differs from the already loaded namespace
+                    warn("Ignoring cached namespace '%s' version %s because version %s is already loaded."
+                         % (ns['name'], ns['version'], self.__namespaces.get(ns['name'])['version']))
             else:
                 to_load.append(ns)
         # now load specs into namespace
