@@ -1,15 +1,17 @@
-import copy as _copy
-from abc import ABCMeta
 import collections
+import copy as _copy
+import types
+import warnings
+from abc import ABCMeta
+from enum import Enum
+
 import h5py
 import numpy as np
-import warnings
-import types
-from enum import Enum
 
 __macros = {
     'array_data': [np.ndarray, list, tuple, h5py.Dataset],
     'scalar_data': [str, int, float, bytes],
+    'data': []
 }
 
 # code to signify how to handle positional arguments in docval
@@ -27,20 +29,35 @@ if hasattr(np, "longdouble"):  # pragma: no cover
     __supported_float_types.append(np.longdouble)
 __supported_float_types = tuple(__supported_float_types)
 __allowed_enum_types = (__supported_bool_types + __supported_uint_types + __supported_int_types
-                        + __supported_float_types + (str, ))
+                        + __supported_float_types + (str,))
 
 
 def docval_macro(macro):
     """Class decorator to add the class to a list of types associated with the key macro in the __macros dict
     """
+
     def _dec(cls):
         if macro not in __macros:
             __macros[macro] = list()
         __macros[macro].append(cls)
         return cls
+
     return _dec
 
 
+def get_docval_macro(key=None):
+    """
+    Return a deepcopy of the docval macros, i.e., strings that represent a customizable list of types for use in docval.
+
+    :param key: Name of the macro. If key=None, then a dictionary of all macros is returned. Otherwise, a tuple of
+                the types associated with the key is returned.
+    """
+    if key is None:
+        return _copy.deepcopy(__macros)
+    else:
+        return tuple(__macros[key])
+
+
 def __type_okay(value, argtype, allow_none=False):
     """Check a value against a type
 
@@ -81,7 +98,7 @@ def __type_okay(value, argtype, allow_none=False):
         return isinstance(value, argtype)
     elif isinstance(argtype, tuple) or isinstance(argtype, list):
         return any(__type_okay(value, i) for i in argtype)
-    else:    # argtype is None
+    else:  # argtype is None
         return True
 
 
@@ -362,6 +379,7 @@ def get_docval(func, *args):
             raise ValueError('Function %s has no docval arguments' % func.__name__)
         return tuple()
 
+
 # def docval_wrap(func, is_method=True):
 #    if is_method:
 #        @docval(*get_docval(func))
@@ -523,7 +541,7 @@ def docval(*validator, **options):  # noqa: C901
                 kw.append(a)
             else:
                 pos.append(a)
-        loc_val = pos+kw
+        loc_val = pos + kw
         _docval[__docval_args_loc] = loc_val
 
         def _check_args(args, kwargs):
@@ -531,13 +549,14 @@ def docval(*validator, **options):  # noqa: C901
             # this function was separated from func_call() in order to make stepping through lines of code using pdb
             # easier
             parsed = __parse_args(
-                        loc_val,
-                        args[1:] if is_method else args,
-                        kwargs,
-                        enforce_type=enforce_type,
-                        enforce_shape=enforce_shape,
-                        allow_extra=allow_extra,
-                        allow_positional=allow_positional)
+                loc_val,
+                args[1:] if is_method else args,
+                kwargs,
+                enforce_type=enforce_type,
+                enforce_shape=enforce_shape,
+                allow_extra=allow_extra,
+                allow_positional=allow_positional
+            )
 
             parse_warnings = parsed.get('future_warnings')
             if parse_warnings:
@@ -575,6 +594,7 @@ def docval(*validator, **options):  # noqa: C901
         setattr(func_call, docval_idx_name, docval_idx)
         setattr(func_call, '__module__', func.__module__)
         return func_call
+
     return dec
 
 
@@ -592,6 +612,7 @@ def __sig_arg(argval):
 
 def __builddoc(func, validator, docstring_fmt, arg_fmt, ret_fmt=None, returns=None, rtype=None):
     '''Generate a Spinxy docstring'''
+
     def to_str(argtype):
         if isinstance(argtype, type):
             module = argtype.__module__
@@ -757,6 +778,7 @@ def get_data_shape(data, strict_no_data_load=False):
     :return: Tuple of ints indicating the size of known dimensions. Dimensions for which the size is unknown
              will be set to None.
     """
+
     def __get_shape_helper(local_data):
         shape = list()
         if hasattr(local_data, '__len__'):
@@ -790,6 +812,25 @@ def pystr(s):
         return s
 
 
+def to_uint_array(arr):
+    """
+    Convert a numpy array or array-like object to a numpy array of unsigned integers with the same dtype itemsize.
+
+    For example, a list of int32 values is converted to a numpy array with dtype uint32.
+    :raises ValueError: if input array contains values that are not unsigned integers or non-negative integers.
+    """
+    if not isinstance(arr, np.ndarray):
+        arr = np.array(arr)
+    if np.issubdtype(arr.dtype, np.unsignedinteger):
+        return arr
+    if np.issubdtype(arr.dtype, np.integer):
+        if (arr < 0).any():
+            raise ValueError('Cannot convert negative integer values to uint.')
+        dt = np.dtype('uint' + str(int(arr.dtype.itemsize*8)))  # keep precision
+        return arr.astype(dt)
+    raise ValueError('Cannot convert array of dtype %s to uint.' % arr.dtype)
+
+
 class LabelledDict(dict):
     """A dict wrapper that allows querying by an attribute of the values and running a callable on removed items.
 
