@@ -22,6 +22,18 @@ class DtypeSpecHelper(TestCase):
         result = DtypeHelper.simplify_cpd_type(compound_type)
         self.assertListEqual(result, expected_result)
 
+    def test_check_dtype_ok(self):
+        self.assertEqual('int', DtypeHelper.check_dtype('int'))
+
+    def test_check_dtype_bad(self):
+        msg = "dtype 'bad dtype' is not a valid primary data type."
+        with self.assertRaisesRegex(ValueError, msg):
+            DtypeHelper.check_dtype('bad dtype')
+
+    def test_check_dtype_ref(self):
+        refspec = RefSpec(target_type='target', reftype='object')
+        self.assertIs(refspec, DtypeHelper.check_dtype(refspec))
+
 
 class DtypeSpecTests(TestCase):
     def setUp(self):
@@ -40,19 +52,19 @@ class DtypeSpecTests(TestCase):
         self.assertEqual(spec.dtype, 'int')
 
     def test_invalid_refspec_dict(self):
-        with self.assertRaises(AssertionError):
-            DtypeSpec.assertValidDtype({'no target': 'test',   # <-- missing or here bad target key for RefSpec
-                                        'reftype': 'object'})
+        """Test missing or bad target key for RefSpec."""
+        msg = "'dtype' must have the key 'target_type'"
+        with self.assertRaisesWith(ValueError, msg):
+            DtypeSpec.assertValidDtype({'no target': 'test', 'reftype': 'object'})
 
     def test_refspec_dtype(self):
         # just making sure this does not cause an error
         DtypeSpec('column1', 'an example column', RefSpec('TimeSeries', 'object'))
 
     def test_invalid_dtype(self):
-        with self.assertRaises(AssertionError):
-            DtypeSpec('column1', 'an example column',
-                      dtype='bad dtype'                     # <-- make sure a bad type string raises an error
-                      )
+        msg = "dtype 'bad dtype' is not a valid primary data type."
+        with self.assertRaisesRegex(ValueError, msg):
+            DtypeSpec('column1', 'an example column', dtype='bad dtype')
 
     def test_is_ref(self):
         spec = DtypeSpec('column1', 'an example column', RefSpec('TimeSeries', 'object'))
