@@ -1,7 +1,7 @@
 import numpy as np
-
-from hdmf.utils import docval, fmt_docval_args, get_docval, getargs, popargs, AllowPositional
 from hdmf.testing import TestCase
+from hdmf.utils import (docval, fmt_docval_args, get_docval, getargs, popargs, AllowPositional, get_docval_macro,
+                        docval_macro)
 
 
 class MyTestClass(object):
@@ -968,3 +968,24 @@ class TestPopargs(TestCase):
         msg = "Argument not found in dict: 'c'"
         with self.assertRaisesWith(ValueError, msg):
             popargs('a', 'c', kwargs)
+
+
+class TestMacro(TestCase):
+
+    def test_macro(self):
+        self.assertTrue(isinstance(get_docval_macro(), dict))
+        self.assertSetEqual(set(get_docval_macro().keys()), {'array_data', 'scalar_data', 'data'})
+
+        self.assertTupleEqual(get_docval_macro('scalar_data'), (str, int, float, bytes))
+
+        @docval_macro('scalar_data')
+        class Dummy1:
+            pass
+
+        self.assertTupleEqual(get_docval_macro('scalar_data'), (str, int, float, bytes, Dummy1))
+
+        @docval_macro('dummy')
+        class Dummy2:
+            pass
+
+        self.assertTupleEqual(get_docval_macro('dummy'), (Dummy2, ))
